from io import BytesIO
from typing import Optional, Union, overload

from faker.providers import BaseProvider
from odf import draw, style
from odf.opendocument import OpenDocumentPresentation
from odf.text import P

from ..base import BytesValue, FileMixin, StringValue
from ..constants import DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("OdpFileProvider",)


class OdpFileProvider(BaseProvider, FileMixin):
    """ODP file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.odp_file import OdpFileProvider

        FAKER = Faker()

        file = OdpFileProvider(FAKER).odp_file()

    Usage example with options:

        file = OdpFileProvider(FAKER).odp_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = OdpFileProvider(FAKER).odp_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "odp"

    @overload
    def odp_file(
        self: "OdpFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def odp_file(
        self: "OdpFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def odp_file(
        self: "OdpFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate an ODP file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )

        data = {"content": content, "filename": filename}

        _fake_file = BytesIO()
        pres_doc = OpenDocumentPresentation()
        # We must describe the dimensions of the page
        page_layout = style.PageLayout(name="MyLayout")
        pres_doc.automaticstyles.addElement(page_layout)
        page_layout.addElement(
            style.PageLayoutProperties(
                margin="0cm",
                pagewidth="28cm",
                pageheight="21cm",
                printorientation="landscape",
            )
        )

        # Every drawing page must have a master page assigned to it.
        master_page = style.MasterPage(
            name="MyMaster", pagelayoutname=page_layout
        )
        pres_doc.masterstyles.addElement(master_page)

        # Style for the title frame of the page
        # We set a centered 34pt font with yellowish background
        title_style = style.Style(name="MyMaster-title", family="presentation")
        title_style.addElement(style.ParagraphProperties(textalign="center"))
        title_style.addElement(style.TextProperties(fontsize="12pt"))
        title_style.addElement(style.GraphicProperties(fillcolor="#ffff99"))
        pres_doc.styles.addElement(title_style)

        # Style for the photo frame
        main_style = style.Style(name="MyMaster-main", family="presentation")
        pres_doc.styles.addElement(main_style)

        # Create style for drawing page
        dp_style = style.Style(name="dp1", family="drawing-page")
        pres_doc.automaticstyles.addElement(dp_style)

        page = draw.Page(stylename=dp_style, masterpagename=master_page)
        pres_doc.presentation.addElement(page)

        title_frame = draw.Frame(
            stylename=title_style,
            width="720pt",
            height="56pt",
            x="40pt",
            y="10pt",
        )
        page.addElement(title_frame)
        textbox = draw.TextBox()
        title_frame.addElement(textbox)
        textbox.addElement(P(text=content))
        pres_doc.save(_fake_file)

        if raw:
            raw_content = BytesValue(_fake_file.getvalue())
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, _fake_file.getvalue())

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import viewsets
from rest_framework.permissions import AllowAny

from ..models import Upload
from .serializers import UploadInSerializer, UploadOutSerializer

__all__ = ("UploadViewSet",)


@extend_schema(tags=[_("Upload")])
@extend_schema_view(
    create=extend_schema(
        tags=[_("Upload")],
        request={
            "multipart/form-data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "format": "string",
                    },
                    "description": {
                        "type": "string",
                        "format": "string",
                    },
                    "file": {
                        "type": "string",
                        "format": "binary",
                    },
                },
            }
        },
    ),
)
class UploadViewSet(viewsets.ModelViewSet):
    queryset = Upload.objects.all()
    serializer_class = default_serializer_class = UploadInSerializer
    serializer_classes = {
        "list": UploadOutSerializer,
        "create": UploadInSerializer,
        "retrieve": UploadOutSerializer,
        "update": UploadOutSerializer,
        "partial_update": UploadOutSerializer,
        "destroy": UploadOutSerializer,
    }
    permission_classes = (AllowAny,)

    def get_serializer_class(self):
        return self.serializer_classes.get(
            self.action, self.default_serializer_class
        )



import logging
from io import BytesIO

from reportlab.lib.pagesizes import letter
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import inch
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import Paragraph, SimpleDocTemplate

from ....constants import DEFAULT_FONT_NAME, DEFAULT_FONT_PATH
from ...base.pdf_generator import BasePdfGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("ReportlabPdfGenerator",)

LOGGER = logging.getLogger(__name__)


class ReportlabPdfGenerator(BasePdfGenerator):
    """Reportlab PDF generator.

    Usage example:

        from faker import Faker
        from faker_file.providers.pdf_file import PdfFileProvider
        from faker_file.providers.pdf_file.generators import reportlab_generator

        FAKER = Faker()

        file = PdfFileProvider(FAKER).pdf_file(
            pdf_generator_cls=reportlab_generator.ReportlabPdfGenerator
        )
    """

    font_name: str = DEFAULT_FONT_NAME
    font_path: str = DEFAULT_FONT_PATH

    def handle_kwargs(self: "ReportlabPdfGenerator", **kwargs) -> None:
        """Handle kwargs."""
        if "font_name" in kwargs:
            self.font_name = kwargs["font_name"]
        if "font_path" in kwargs:
            self.font_path = kwargs["font_path"]

    def generate(
        self: "ReportlabPdfGenerator", content: str, **kwargs
    ) -> bytes:
        """Generate PDF."""
        styles = getSampleStyleSheet()
        style_paragraph = styles["Normal"]
        style_paragraph.fontName = self.font_name
        pdfmetrics.registerFont(TTFont(self.font_name, self.font_path))

        story = []
        buffer = BytesIO()
        doc = SimpleDocTemplate(
            buffer,
            pagesize=letter,
            bottomMargin=0.4 * inch,
            topMargin=0.6 * inch,
            rightMargin=0.8 * inch,
            leftMargin=0.8 * inch,
        )
        paragraph = Paragraph(content, style_paragraph)
        story.append(paragraph)
        doc.build(story)

        return buffer.getvalue()



__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "DEFAULT_AUDIO_MAX_NB_CHARS",
    "DEFAULT_FILE_ENCODING",
    "DEFAULT_IMAGE_MAX_NB_CHARS",
    "DEFAULT_TEXT_CONTENT_TEMPLATE",
    "DEFAULT_TEXT_MAX_NB_CHARS",
    "DEFAULT_FONT_NAME",
    "DEFAULT_FONT_PATH",
)

DEFAULT_AUDIO_MAX_NB_CHARS = 500
DEFAULT_FILE_ENCODING = "utf-8"
DEFAULT_IMAGE_MAX_NB_CHARS = 5_000
DEFAULT_TEXT_CONTENT_TEMPLATE = """{{text}}"""
DEFAULT_TEXT_MAX_NB_CHARS = 10_000
DEFAULT_FONT_NAME = "Vera"
DEFAULT_FONT_PATH = "Vera.ttf"



from typing import Any, Dict, Optional, Type, Union, overload

from faker.providers import BaseProvider

from ...base import BytesValue, FileMixin, StringValue
from ...constants import DEFAULT_TEXT_MAX_NB_CHARS
from ...storages.base import BaseStorage
from ...storages.filesystem import FileSystemStorage
from ..base.pdf_generator import BasePdfGenerator
from .generators.pdfkit_generator import PdfkitPdfGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PdfFileProvider",)


class PdfFileProvider(BaseProvider, FileMixin):
    """PDF file provider.

        Usage example:

        from faker_file.providers.pdf_file import PdfFileProvider

        file = PdfFileProvider(None).pdf_file()

    Usage example with options:

        from faker_file.providers.pdf_file import PdfFileProvider

        file = PdfFileProvider(None).pdf_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = PdfFileProvider(Faker()).pdf_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Default PDF generator class is `PdfkitPdfGenerator` which uses `pdfkit`
    Python package and `wkhtmltopdf` system package for generating PDFs from
    randomly generated text. The quality of the produced PDFs is very good,
    but it's less performant than `ReportlabPdfGenerator` (factor 40x), which
    does not require additional system dependencies to run. To use it, pass
    `ReportlabPdfGenerator` class in `pdf_generator_cls` argument.

        from faker_file.providers.pdf_file.generators import (
            reportlab_generator,
        )

        file = PdfFileProvider(None).pdf_file(
            max_nb_chars=1_000,
            wrap_chars_after=80,
            pdf_generator_cls=reportlab_generator.ReportlabPdfGenerator,
        )
    """

    extension: str = "pdf"

    @overload
    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        pdf_generator_cls: Optional[Type[BasePdfGenerator]] = (
            PdfkitPdfGenerator
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        pdf_generator_cls: Optional[Type[BasePdfGenerator]] = (
            PdfkitPdfGenerator
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        pdf_generator_cls: Optional[Type[BasePdfGenerator]] = (
            PdfkitPdfGenerator
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a PDF file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param pdf_generator_cls: PDF generator class.
        :param pdf_generator_kwargs: PDF generator kwargs.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )

        data = {"content": content, "filename": filename}

        if pdf_generator_cls is None:
            pdf_generator_cls = PdfkitPdfGenerator

        if not pdf_generator_kwargs:
            pdf_generator_kwargs = {}
        pdf_generator = pdf_generator_cls(
            generator=self.generator,
            **pdf_generator_kwargs,
        )
        _raw_content = pdf_generator.generate(
            content=content,
        )

        if raw:
            raw_content = BytesValue(_raw_content)
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, _raw_content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



from random import choice
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    overload,
)

from faker import Faker
from faker.generator import Generator
from faker.providers.python import Provider

from ...base import BytesValue, StringValue
from ...constants import (
    DEFAULT_AUDIO_MAX_NB_CHARS,
    DEFAULT_IMAGE_MAX_NB_CHARS,
    DEFAULT_TEXT_MAX_NB_CHARS,
)
from ...storages.base import BaseStorage
from ..base.mp3_generator import BaseMp3Generator
from ..base.pdf_generator import BasePdfGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "create_inner_bin_file",
    "create_inner_csv_file",
    "create_inner_docx_file",
    "create_inner_eml_file",
    "create_inner_epub_file",
    "create_inner_ico_file",
    "create_inner_jpeg_file",
    "create_inner_mp3_file",
    "create_inner_odp_file",
    "create_inner_ods_file",
    "create_inner_odt_file",
    "create_inner_pdf_file",
    "create_inner_png_file",
    "create_inner_pptx_file",
    "create_inner_rtf_file",
    "create_inner_svg_file",
    "create_inner_tar_file",
    "create_inner_txt_file",
    "create_inner_webp_file",
    "create_inner_xlsx_file",
    "create_inner_zip_file",
    "fuzzy_choice_create_inner_file",
)


# ************************************************
# ********************* BIN **********************
# ************************************************


@overload
def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner BIN file."""
    try:
        from ..bin_file import BinFileProvider
    except ImportError as err:
        raise err

    return BinFileProvider(generator).bin_file(
        storage=storage,
        prefix=prefix,
        length=length,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* CSV **********************
# ************************************************


@overload
def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner CSV file."""
    try:
        from ..csv_file import CsvFileProvider
    except ImportError as err:
        raise err

    return CsvFileProvider(generator).csv_file(
        storage=storage,
        prefix=prefix,
        header=header,
        data_columns=data_columns,
        num_rows=num_rows,
        include_row_ids=include_row_ids,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** DOCX **********************
# ************************************************


@overload
def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner DOCX file."""
    try:
        from ..docx_file import DocxFileProvider
    except ImportError as err:
        raise err

    return DocxFileProvider(generator).docx_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* EML **********************
# ************************************************


@overload
def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner EML file."""
    try:
        from ..eml_file import EmlFileProvider
    except ImportError as err:
        raise err
    return EmlFileProvider(generator).eml_file(
        storage=storage,
        prefix=prefix,
        options=options,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** EPUB **********************
# ************************************************


@overload
def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner EPUB file."""
    try:
        from ..epub_file import EpubFileProvider
    except ImportError as err:
        raise err

    return EpubFileProvider(generator).epub_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        title=title,
        chapter_title=chapter_title,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ICO **********************
# ************************************************


@overload
def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ICO file."""
    try:
        from ..ico_file import IcoFileProvider
    except ImportError as err:
        raise err

    return IcoFileProvider(generator).ico_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** JPEG **********************
# ************************************************


@overload
def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner JPEG file."""
    try:
        from ..jpeg_file import JpegFileProvider
    except ImportError as err:
        raise err

    return JpegFileProvider(generator).jpeg_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* MP3 **********************
# ************************************************


@overload
def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Type[BaseMp3Generator]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Type[BaseMp3Generator]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Type[BaseMp3Generator]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODS file."""
    try:
        from ..mp3_file import Mp3FileProvider
    except ImportError as err:
        raise err

    return Mp3FileProvider(generator).mp3_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        content=content,
        mp3_generator_cls=mp3_generator_cls,
        mp3_generator_kwargs=mp3_generator_kwargs,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODP **********************
# ************************************************


@overload
def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    ...


def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODP file."""
    try:
        from ..odp_file import OdpFileProvider
    except ImportError as err:
        raise err

    return OdpFileProvider(generator).odp_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODS **********************
# ************************************************


@overload
def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODS file."""
    try:
        from ..ods_file import OdsFileProvider
    except ImportError as err:
        raise err

    return OdsFileProvider(generator).ods_file(
        storage=storage,
        prefix=prefix,
        data_columns=data_columns,
        num_rows=num_rows,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODT **********************
# ************************************************


@overload
def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODT file."""
    try:
        from ..odt_file import OdtFileProvider
    except ImportError as err:
        raise err

    return OdtFileProvider(generator).odt_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* PDF **********************
# ************************************************


@overload
def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Type[BasePdfGenerator]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Type[BasePdfGenerator]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Type[BasePdfGenerator]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PDF file."""
    try:
        from ..pdf_file import PdfFileProvider
    except ImportError as err:
        raise err

    return PdfFileProvider(generator).pdf_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        pdf_generator_cls=pdf_generator_cls,
        pdf_generator_kwargs=pdf_generator_kwargs,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* PNG **********************
# ************************************************


@overload
def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PNG file."""
    try:
        from ..png_file import PngFileProvider
    except ImportError as err:
        raise err

    return PngFileProvider(generator).png_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** PPTX **********************
# ************************************************


@overload
def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PPTX file."""
    try:
        from ..pptx_file import PptxFileProvider
    except ImportError as err:
        raise err

    return PptxFileProvider(generator).pptx_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* RTF **********************
# ************************************************


@overload
def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner RTF file."""
    try:
        from ..rtf_file import RtfFileProvider
    except ImportError as err:
        raise err

    return RtfFileProvider(generator).rtf_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* SVG **********************
# ************************************************


@overload
def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner SVG file."""
    try:
        from ..svg_file import SvgFileProvider
    except ImportError as err:
        raise err

    return SvgFileProvider(generator).svg_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* TAR **********************
# ************************************************


@overload
def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner TAR file."""
    try:
        from ..tar_file import TarFileProvider
    except ImportError as err:
        raise err
    return TarFileProvider(generator).tar_file(
        storage=storage,
        prefix=prefix,
        options=options,
        compression=compression,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* TXT **********************
# ************************************************


@overload
def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner TXT file."""
    try:
        from ..txt_file import TxtFileProvider
    except ImportError as err:
        raise err

    return TxtFileProvider(generator).txt_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** WEBP **********************
# ************************************************


@overload
def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner WEBP file."""
    try:
        from ..webp_file import WebpFileProvider
    except ImportError as err:
        raise err

    return WebpFileProvider(generator).webp_file(
        storage=storage,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** XLSX **********************
# ************************************************


@overload
def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner XLSX file."""
    try:
        from ..xlsx_file import XlsxFileProvider
    except ImportError as err:
        raise err

    return XlsxFileProvider(generator).xlsx_file(
        storage=storage,
        prefix=prefix,
        data_columns=data_columns,
        num_rows=num_rows,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ZIP **********************
# ************************************************


@overload
def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue:
    ...


@overload
def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> StringValue:
    ...


def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ZIP file."""
    try:
        from ..zip_file import ZipFileProvider
    except ImportError as err:
        raise err
    return ZipFileProvider(generator).zip_file(
        storage=storage,
        prefix=prefix,
        options=options,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** OTHER *********************
# ************************************************


def fuzzy_choice_create_inner_file(
    func_choices: List[Tuple[Callable, Dict[str, Any]]],
    **kwargs,
) -> StringValue:
    """Create inner file from given list of function choices.

    :param func_choices: List of functions to choose from.
    :return: StringValue.

    Usage example:

        from faker import Faker
        from faker_file.providers.helpers.inner import (
            create_inner_docx_file,
            create_inner_epub_file,
            create_inner_txt_file,
            fuzzy_choice_create_inner_file,
        )
        from faker_file.storages.filesystem import FileSystemStorage

        FAKER = Faker()
        STORAGE = FileSystemStorage()

        kwargs = {"storage": STORAGE, "generator": FAKER}
        file = fuzzy_choice_create_inner_file(
            [
                (create_inner_docx_file, kwargs),
                (create_inner_epub_file, kwargs),
                (create_inner_txt_file, kwargs),
            ]
        )

    You could use it in archives to make a variety of different file types
    within the archive.

        from faker import Faker
        from faker_file.providers.helpers.inner import (
            create_inner_docx_file,
            create_inner_epub_file,
            create_inner_txt_file,
            fuzzy_choice_create_inner_file,
        )
        from faker_file.providers.zip_file import ZipFileProvider
        from faker_file.storages.filesystem import FileSystemStorage

        FAKER = Faker()
        STORAGE = FileSystemStorage()

        kwargs = {"storage": STORAGE, "generator": FAKER}
        file = ZipFileProvider(FAKER).zip_file(
            prefix="zzz_archive_",
            options={
                "count": 50,
                "create_inner_file_func": fuzzy_choice_create_inner_file,
                "create_inner_file_args": {
                    "func_choices": [
                        (create_inner_docx_file, kwargs),
                        (create_inner_epub_file, kwargs),
                        (create_inner_txt_file, kwargs),
                    ],
                },
                "directory": "zzz",
            }
        )
    """
    _func, _kwargs = choice(func_choices)
    return _func(**_kwargs)



from typing import Optional, Union, overload

from faker.providers import BaseProvider

from ..base import BytesValue, StringValue
from ..constants import DEFAULT_IMAGE_MAX_NB_CHARS
from ..storages.base import BaseStorage
from .mixins.image_mixin import ImageMixin

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PngFileProvider",)


class PngFileProvider(BaseProvider, ImageMixin):
    """PNG file provider.

        Usage example:

        from faker import Faker
        from faker_file.providers.png_file import PngFileProvider

        file = PngFileProvider(Faker()).png_file()

    Usage example with options:

        file = PngFileProvider(Faker()).png_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = PngFileProvider(Faker()).png_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "png"

    @overload
    def png_file(
        self: "PngFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def png_file(
        self: "PngFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def png_file(
        self: "PngFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a PNG file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        return self._image_file(
            storage=storage,
            prefix=prefix,
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
            raw=raw,
            **kwargs,
        )



# set optional bootswatch theme
# see http://bootswatch.com/3/ for available swatches
FLASK_ADMIN_SWATCH = "cerulean"

# Create dummy secrey key so we can use sessions
SECRET_KEY = "123456790"

# Create in-memory database
DATABASE_FILE = "sample_db_test.sqlite"
SQLALCHEMY_DATABASE_URI = "sqlite://"
SQLALCHEMY_ECHO = True
SQLALCHEMY_TRACK_MODIFICATIONS = False



from typing import Any, Callable, Dict, List, Optional, Union

from faker import Faker
from faker.generator import Generator
from faker.providers.python import Provider

from .helpers import wrap_text

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "BytesValue",
    "DEFAULT_REL_PATH",
    "FileMixin",
    "StringValue",
)


DEFAULT_REL_PATH = "tmp"


class StringValue(str):
    data: Dict[str, Any] = {}


class BytesValue(bytes):
    data: Dict[str, Any] = {}


class FileMixin:
    """File mixin."""

    numerify: Callable
    random_element: Callable
    formats: List[str]
    generator: Union[Faker, Generator, Provider]
    extension: str  # Desired file extension.

    def _generate_text_content(
        self,
        max_nb_chars: int,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
    ) -> str:
        """Generate text content.

        :param max_nb_chars: Max number of chars.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :return: String with random chars.
        """
        if self.generator is None:
            self.generator = Faker()

        # Specific
        if content is None:
            content = self.generator.text(max_nb_chars)
        else:
            content = self.generator.pystr_format(content)

        if wrap_chars_after:
            content = wrap_text(content, wrap_chars_after)

        return content



"""
Django settings for simple project.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

from .core import PROJECT_DIR

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

try:
    from .local_settings import DEBUG_TEMPLATE
except ImportError:
    DEBUG_TEMPLATE = False

try:
    from .local_settings import USE_CACHED_TEMPLATE_LOADERS
except ImportError:
    USE_CACHED_TEMPLATE_LOADERS = False

if USE_CACHED_TEMPLATE_LOADERS:
    _TEMPLATE_LOADERS = [
        (
            "django.template.loaders.cached.Loader",
            (
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ),
        ),
    ]
else:
    _TEMPLATE_LOADERS = [
        "django.template.loaders.filesystem.Loader",
        "django.template.loaders.app_directories.Loader",
    ]


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # 'APP_DIRS': True,
        "DIRS": [PROJECT_DIR(os.path.join("..", "templates"))],
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "loaders": _TEMPLATE_LOADERS,
            "debug": DEBUG_TEMPLATE,
        },
    }
]
# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "uzc&9xi6b#dz^z7tpa+br3ohq)-9%v9ux@9^t!(5fl41n%&mn$"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
DEV = False

ALLOWED_HOSTS = ["*"]

# Application definition

INSTALLED_APPS = (
    # Django standard
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # Handy apps
    # REST framework specific
    "drf_spectacular",
    "rest_framework",
    # Example app
    "upload",
)

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    # 'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

ROOT_URLCONF = "project.urls"

WSGI_APPLICATION = "project.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = "en-US"

TIME_ZONE = "UTC"

USE_I18N = True

# USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = "/static/"
MEDIA_URL = "/media/"
STATIC_ROOT = PROJECT_DIR(os.path.join(BASE_DIR, "static"))
MEDIA_ROOT = PROJECT_DIR(os.path.join(BASE_DIR, "media"))

REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": (
        "rest_framework.permissions.IsAuthenticated",
    ),
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_PAGINATION_CLASS": (
        "rest_framework.pagination.PageNumberPagination"
    ),
    "PAGE_SIZE": 100,
}

# A sample logging configuration.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"}
    },
    "root": {
        "level": "INFO",
        "handlers": ["console"],
    },
    "formatters": {
        "verbose": {
            "format": "\n%(levelname)s %(asctime)s [%(pathname)s:%(lineno)s] "
            "%(message)s"
        },
    },
    "handlers": {
        "console": {
            "level": "INFO",
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
    },
    "loggers": {
        "": {
            "handlers": ["console"],
            "level": "DEBUG",
            "propagate": False,
        },
    },
}

# Do not put any settings below this line
try:
    from .local_settings import *  # noqa
except ImportError:
    pass



import os
from pathlib import Path
from random import choice
from typing import Optional, Union, overload

from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("RandomFileFromDirProvider",)


class RandomFileFromDirProvider(BaseProvider, FileMixin):
    """Random file from given directory provider.

    Usage example:

        from faker_file.providers.random_file_from_dir import (
            RandomFileFromDirProvider,
        )

        file = RandomFileFromDirProvider(None).random_file_from_dir(
            source_dir_path="/tmp/tmp/",
        )

    Usage example with options:

        from faker_file.providers.random_file_from_dir import (
            RandomFileFromDirProvider,
        )

        file = RandomFileFromDirProvider(None).random_file_from_dir(
            source_dir_path="/tmp/tmp/",
            prefix="zzz",
        )
    """

    extension: str = ""

    @overload
    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Pick a random file from given directory.

        :param source_dir_path: Source files directory.
        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        # Specific
        source_file_choices = [
            os.path.join(source_dir_path, _f)
            for _f in os.listdir(source_dir_path)
            if os.path.isfile(os.path.join(source_dir_path, _f))
        ]
        source_file_path = choice(source_file_choices)
        source_file = Path(source_file_path)

        # Generic
        filename = storage.generate_filename(
            prefix=prefix,
            extension=source_file.suffix[1:],
        )

        # Specific
        with open(source_file_path, "rb") as _file:
            if raw:
                raw_content = BytesValue(_file.read())
                return raw_content

            storage.write_bytes(filename, _file.read())

        # Generic
        file_name = StringValue(storage.relpath(filename))
        return file_name



import os
import zipfile
from io import BytesIO
from pathlib import Path
from typing import Any, Dict, Optional, Union, overload

from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage
from .helpers.inner import create_inner_txt_file

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("ZipFileProvider",)


class ZipFileProvider(BaseProvider, FileMixin):
    """ZIP file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.zip_file import ZipFileProvider

        FAKER = Faker()

        file = ZipFileProvider(FAKER).zip_file()

    Usage example with options:

        from faker_file.providers.helpers.inner import create_inner_docx_file
        from faker_file.providers.zip_file import ZipFileProvider

        file = ZipFileProvider(FAKER).zip_file(
            prefix="zzz_archive_",
            options={
                "count": 5,
                "create_inner_file_func": create_inner_docx_file,
                "create_inner_file_args": {
                    "prefix": "zzz_docx_file_",
                    "max_nb_chars": 1_024,
                },
                "directory": "zzz",
            },
        )

    Usage example of nested ZIPs:

        from faker_file.providers.helpers.inner import create_inner_zip_file

        file = ZipFileProvider(FAKER).zip_file(
            options={
                "create_inner_file_func": create_inner_zip_file,
                "create_inner_file_args": {
                    "options": {
                        "create_inner_file_func": create_inner_docx_file,
                    },
                },
            },
        )

    If you want to see, which files were included inside the ZIP, check
    the ``file.data["files"]``.
    """

    extension: str = "zip"

    @overload
    def zip_file(
        self: "ZipFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        options: Optional[Dict[str, Any]] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def zip_file(
        self: "ZipFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        options: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        ...

    def zip_file(
        self: "ZipFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        options: Optional[Dict[str, Any]] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a ZIP file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param options: Options (non-structured) for complex types, such as ZIP.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )
        data: Dict[str, Any] = {"inner": {}, "files": [], "filename": filename}
        fs_storage = FileSystemStorage()

        # Specific
        if options:
            """
            A complex case. Could be initialized as follows:

                zip_file = ZipFileProvider(None).zip_file(
                    prefix="zzz_archive_",
                    options={
                        "count": 5,
                        "create_inner_file_func": create_inner_docx_file,
                        "create_inner_file_args": {
                            "prefix": "zzz_file_",
                            "max_nb_chars": 1_024,
                            "content": "{{date}}\r\n{{text}}\r\n{{name}}",
                        },
                        "directory": "zzz",
                    },
                )
            """
            _count = options.get("count", 5)
            _create_inner_file_func = options.get(
                "create_inner_file_func", create_inner_txt_file
            )
            _create_inner_file_args = options.get("create_inner_file_args", {})
            _dir_path = Path("")
            _directory = options.get("directory", "")

        else:
            # Defaults
            _count = 5
            _create_inner_file_func = create_inner_txt_file
            _create_inner_file_args = {}
            _dir_path = Path("")
            _directory = ""

        _zip_content = BytesIO()
        with zipfile.ZipFile(_zip_content, "w") as __fake_file:
            _kwargs = {"generator": self.generator}
            _kwargs.update(_create_inner_file_args)
            for __i in range(_count):
                __file = _create_inner_file_func(
                    storage=fs_storage,
                    **_kwargs,
                )
                data["inner"][str(__file)] = __file
                __file_abs_path = fs_storage.abspath(__file)
                __fake_file.write(
                    __file_abs_path,
                    arcname=Path(_directory) / Path(__file).name,
                )
                os.remove(__file_abs_path)  # Clean up temporary files
                data["files"].append(Path(_directory) / Path(__file).name)

        if raw:
            raw_content = BytesValue(_zip_content.getvalue())
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, _zip_content.getvalue())

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



from google.oauth2 import service_account
from pathy import set_client_params

from .cloud import CloudStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("GoogleCloudStorage",)


class GoogleCloudStorage(CloudStorage):
    """Google Cloud Storage.

    Usage example:

        from faker_file.storages.google_cloud_storage import GoogleCloudStorage

        gs_storage = GoogleCloudStorage(
            bucket_name="artur-testing-1",
            rel_path="tmp",
        )
        file = gs_storage.generate_filename(prefix="zzz_", extension="docx")
        gs_storage.write_text(file, "Lorem ipsum")
        gs_storage.write_bytes(file, b"Lorem ipsum")
    """

    schema = "gs"

    def authenticate(
        self: "GoogleCloudStorage", json_file_path: str, **kwargs
    ) -> None:
        """Authenticate to Google Cloud Storage."""
        credentials = service_account.Credentials.from_service_account_file(
            json_file_path
        )
        set_client_params("gs", credentials=credentials)



__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "TEXT_DOCX",
    "TEXT_EML",
    "TEXT_PDF",
    "TEXT_TXT",
    "TEXT_EPUB",
    "TEXT_RTF",
    "TEXTS",
)


TEXT_DOCX = """
“How doth the little crocodile
    Improve his shining tail,
And pour the waters of the Nile
    On every golden scale!

“How cheerfully he seems to grin,
    How neatly spread his claws,
And welcome little fishes in
    With gently smiling jaws!”
"""

TEXT_EML = """
“Will you walk a little faster?” said a whiting to a snail.
“There’s a porpoise close behind us, and he’s treading on my tail.
See how eagerly the lobsters and the turtles all advance!
They are waiting on the shingle—will you come and join the dance?
Will you, won’t you, will you, won’t you, will you join the dance?
Will you, won’t you, will you, won’t you, won’t you join the dance?

“You can really have no notion how delightful it will be
When they take us up and throw us, with the lobsters, out to sea!”
But the snail replied “Too far, too far!” and gave a look askance—
Said he thanked the whiting kindly, but he would not join the dance.
Would not, could not, would not, could not, would not join the dance.
Would not, could not, would not, could not, could not join the dance.

“What matters it how far we go?” his scaly friend replied.
“There is another shore, you know, upon the other side.
The further off from England the nearer is to France—
Then turn not pale, beloved snail, but come and join the dance.
Will you, won’t you, will you, won’t you, will you join the dance?
Will you, won’t you, will you, won’t you, won’t you join the dance?”
"""

TEXT_PDF = """
         “Fury said to a
         mouse, That he
        met in the
       house,
     ‘Let us
      both go to
       law: I will
        prosecute
         you.—Come,
           I’ll take no
           denial; We
          must have a
        trial: For
      really this
     morning I’ve
    nothing
    to do.’
      Said the
      mouse to the
       cur, ‘Such
        a trial,
         dear sir,
            With
          no jury
        or judge,
       would be
      wasting
      our
      breath.’
        ‘I’ll be
        judge, I’ll
         be jury,’
             Said
         cunning
          old Fury:
          ‘I’ll
          try the
            whole
            cause,
              and
           condemn
           you
          to
           death.’”
"""

TEXT_TXT = """
“You are old, Father William,” the young man said,
    “And your hair has become very white;
And yet you incessantly stand on your head—
    Do you think, at your age, it is right?”

“In my youth,” Father William replied to his son,
    “I feared it might injure the brain;
But, now that I’m perfectly sure I have none,
    Why, I do it again and again.”

“You are old,” said the youth, “as I mentioned before,
    And have grown most uncommonly fat;
Yet you turned a back-somersault in at the door—
    Pray, what is the reason of that?”

“In my youth,” said the sage, as he shook his grey locks,
    “I kept all my limbs very supple
By the use of this ointment—one shilling the box—
    Allow me to sell you a couple?”

“You are old,” said the youth, “and your jaws are too weak
    For anything tougher than suet;
Yet you finished the goose, with the bones and the beak—
    Pray, how did you manage to do it?”

“In my youth,” said his father, “I took to the law,
    And argued each case with my wife;
And the muscular strength, which it gave to my jaw,
    Has lasted the rest of my life.”

“You are old,” said the youth, “one would hardly suppose
    That your eye was as steady as ever;
Yet you balanced an eel on the end of your nose—
    What made you so awfully clever?”

“I have answered three questions, and that is enough,”
    Said his father; “don’t give yourself airs!
Do you think I can listen all day to such stuff?
    Be off, or I’ll kick you down stairs!”
"""

TEXT_EPUB = """
“Speak roughly to your little boy,
    And beat him when he sneezes:
He only does it to annoy,
    Because he knows it teases.”
“I speak severely to my boy,
    I beat him when he sneezes;
For he can thoroughly enjoy
    The pepper when he pleases!”
"""

TEXT_RTF = """
“’Tis the voice of the Lobster; I heard him declare,
“You have baked me too brown, I must sugar my hair.”
As a duck with its eyelids, so he with his nose
Trims his belt and his buttons, and turns out his toes.”

[later editions continued as follows
When the sands are all dry, he is gay as a lark,
And will talk in contemptuous tones of the Shark,
But, when the tide rises and sharks are around,
His voice has a timid and tremulous sound.]

“I passed by his garden, and marked, with one eye,
How the Owl and the Panther were sharing a pie—”

[later editions continued as follows
The Panther took pie-crust, and gravy, and meat,
While the Owl had the dish as its share of the treat.
When the pie was all finished, the Owl, as a boon,
Was kindly permitted to pocket the spoon:
While the Panther received knife and fork with a growl,
And concluded the banquet—]
"""

TEXTS = {
    "docx": TEXT_DOCX,
    "eml": TEXT_EML,
    "epub": TEXT_EPUB,
    "pdf": TEXT_PDF,
    "rtf": TEXT_RTF,
    "txt": TEXT_TXT,
}



from rest_framework import serializers

from ..models import Upload

__all__ = (
    "UploadInSerializer",
    "UploadOutSerializer",
)


class UploadInSerializer(serializers.ModelSerializer):
    class Meta:
        model = Upload
        fields = ["id", "name", "description", "file"]
        read_only_fields = ["id"]


class UploadOutSerializer(serializers.ModelSerializer):
    class Meta:
        model = Upload
        fields = ["id", "name", "description", "file"]



import nlpaug.augmenter.word as naw

from ...base.text_augmenter import BaseTextAugmenter

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "ContextualWordEmbeddingsAugmenter",
    "DEFAULT_ACTION",
    "DEFAULT_MODEL_PATH",
)

DEFAULT_MODEL_PATH = "bert-base-multilingual-cased"
DEFAULT_ACTION = "substitute"


class ContextualWordEmbeddingsAugmenter(BaseTextAugmenter):
    """Text extractor based on `ContextualWordEmbsAug` of `nlpaug`.

    Usage example:

        from faker import Faker
        from faker_file.providers.augment_file_from_dir import (
            AugmentFileFromDirProvider,
        )
        from faker_file.providers.augment_file_from_dir.augmenters import (
            nlpaug_augmenter,
        )

        FAKER = Faker()

        file = AugmentFileFromDirProvider(FAKER).augment_file_from_dir(
            text_augmenter_cls=(
                nlpaug_augmenter.ContextualWordEmbeddingsAugmenter
            ),
            text_augmenter_kwargs={
                "model_path": "bert-base-uncased",
                "action": "substitute",
            }
        )

    Refer to `nlpaug` official documentation and check examples
    for `Textual augmenters`:

        https://nlpaug.readthedocs.io/en/latest/example/example.html

    Some well working options for `model_path` are:

        - bert-base-multilingual-cased
        - bert-base-multilingual-uncased
        - bert-base-cased
        - bert-base-uncased
        - bert-base-german-cased
        - GroNLP/bert-base-dutch-cased

    Options for `action` are:

        - insert
        - substitute
    """

    model_path: str = DEFAULT_MODEL_PATH
    action: str = DEFAULT_ACTION

    def handle_kwargs(
        self: "ContextualWordEmbeddingsAugmenter", **kwargs
    ) -> None:
        """Handle kwargs."""
        if "model_path" in kwargs:
            self.model_path = kwargs["model_path"]
        if "action" in kwargs:
            self.action = kwargs["action"]

    def augment(
        self: "ContextualWordEmbeddingsAugmenter",
        text: str,
    ) -> str:
        """Augment text."""
        aug = naw.ContextualWordEmbsAug(
            model_path=self.model_path,
            action=self.action,
        )
        return aug.augment(text)[0]



import os
import shutil
import tempfile
from typing import Optional, Union, overload

import xml2epub
from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..constants import DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("EpubFileProvider",)


class EpubFileProvider(BaseProvider, FileMixin):
    """EPUB file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.epub_file import EpubFileProvider

        file = EpubFileProvider(Faker()).epub_file()

    Usage example with options:

        file = EpubFileProvider(Faker()).epub_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = EpubFileProvider(Faker()).epub_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "epub"

    @overload
    def epub_file(
        self: "EpubFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        title: Optional[str] = None,
        chapter_title: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def epub_file(
        self: "EpubFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        title: Optional[str] = None,
        chapter_title: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def epub_file(
        self: "EpubFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        title: Optional[str] = None,
        chapter_title: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a EPUB file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param title: E-book title. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param chapter_title: Chapter title. Might contain dynamic elements,
            which are then replaced by correspondent fixtures.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        ).replace("\n", "<br>")

        title = self._generate_text_content(
            max_nb_chars=50,
            content=title,
        )

        chapter_title = self._generate_text_content(
            max_nb_chars=50,
            content=chapter_title,
        )

        _book = xml2epub.Epub(title)
        _chapter = xml2epub.create_chapter_from_string(
            content, title=chapter_title
        )
        _book.add_chapter(_chapter)
        _local_file_name = _book.create_epub(tempfile.gettempdir())

        data = {
            "content": content,
            "title": title,
            "chapter_title": chapter_title,
            "filename": filename,
        }

        _raw_content = bytes()
        with open(_local_file_name, "rb") as fakefile:
            _raw_content = fakefile.read()
            os.remove(_local_file_name)
            shutil.rmtree(_book.EPUB_DIR)

        if raw:
            raw_content = BytesValue(_raw_content)
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, _raw_content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



from pathy import set_client_params

from .cloud import CloudStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("AzureCloudStorage",)


class AzureCloudStorage(CloudStorage):
    """Azure Cloud Storage.

    Usage example:

        from faker_file.storages.azure_cloud_storage import AzureCloudStorage

        azure_storage = AzureCloudStorage(
            bucket_name="artur-testing-1",
            rel_path="tmp",
        )
        file = azure_storage.generate_filename(prefix="zzz_", extension="docx")
        azure_storage.write_text(file, "Lorem ipsum")
        azure_storage.write_bytes(file, b"Lorem ipsum")
    """

    schema = "azure"

    def authenticate(
        self: "AzureCloudStorage", connection_string: str, **kwargs
    ) -> None:
        """Authenticate to Azure Cloud Storage."""
        set_client_params("azure", connection_string=connection_string)



from typing import Any, Optional

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("BaseStorage",)


class BaseStorage:
    """Base storage."""

    def __init__(self, *args, **kwargs) -> None:
        self.args = args
        self.kwargs = kwargs

    def generate_filename(
        self: "BaseStorage", extension: str, prefix: Optional[str] = None
    ) -> Any:
        """Generate filename."""
        raise NotImplementedError(
            "Method generate_filename is not implemented!"
        )

    def write_text(
        self: "BaseStorage",
        filename: Any,
        data: str,
        encoding: Optional[str] = None,
    ) -> int:
        """Write text."""
        raise NotImplementedError("Method write_text is not implemented!")

    def write_bytes(self: "BaseStorage", filename: Any, data: bytes) -> int:
        """Write bytes."""
        raise NotImplementedError("Method write_bytes is not implemented!")

    def exists(self: "BaseStorage", filename: Any) -> bool:
        """Check if file exists."""
        raise NotImplementedError("Method exists is not implemented!")

    def relpath(self: "BaseStorage", filename: Any) -> str:
        """Return relative path."""
        raise NotImplementedError("Method relpath is not implemented!")

    def abspath(self: "BaseStorage", filename: Any) -> str:
        """Return absolute path."""
        raise NotImplementedError("Method abspath is not implemented!")



from typing import Optional, Sequence, Tuple, Union, overload

from faker import Faker
from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("CsvFileProvider",)


FAKER = Faker()


class CsvFileProvider(BaseProvider, FileMixin):
    """CSV file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.csv_file import CsvFileProvider

        file = CsvFileProvider(Faker()).csv_file()

    Usage example with options:

        from faker_file.providers.csv_file import CsvFileProvider

        file = CsvFileProvider(Faker()).csv_file(
            prefix="zzz",
            num_rows=100,
            data_columns=('{{name}}', '{{sentence}}', '{{address}}'),
            include_row_ids=True,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = CsvFileProvider(Faker()).csv_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            num_rows=100,
        )
    """

    extension: str = "csv"

    @overload
    def csv_file(
        self: "CsvFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, ...] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
        content: Optional[str] = None,
        encoding: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def csv_file(
        self: "CsvFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, ...] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
        content: Optional[str] = None,
        encoding: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def csv_file(
        self: "CsvFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        header: Optional[Sequence[str]] = None,
        data_columns: Tuple[str, ...] = ("{{name}}", "{{address}}"),
        num_rows: int = 10,
        include_row_ids: bool = False,
        content: Optional[str] = None,
        encoding: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a CSV file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param header: The ``header`` argument expects a list or a tuple of
            strings that will serve as the header row if supplied.
        :param data_columns: The ``data_columns`` argument expects a list or a
            tuple of string tokens, and these string tokens will be passed to
            :meth:`pystr_format()
            <faker.providers.python.Provider.pystr_format>`
            for data generation. Argument Groups are used to pass arguments
            to the provider methods. Both ``header`` and ``data_columns`` must
            be of the same length.
        :param num_rows: The ``num_rows`` argument controls how many rows of
            data to generate, and the ``include_row_ids`` argument may be set
            to ``True`` to include a sequential row ID column.
        :param include_row_ids:
        :param content: File content. If given, used as is.
        :param encoding: Encoding.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        if self.generator is None:
            self.generator = Faker()

        # Specific
        if content is None:
            content = self.generator.csv(
                header=header,
                data_columns=data_columns,
                num_rows=num_rows,
                include_row_ids=include_row_ids,
            )
        else:
            content = self.generator.pystr_format(content)

        data = {"content": content, "filename": filename}

        if raw:
            raw_content = BytesValue(content.encode("utf8"))
            raw_content.data = data
            return raw_content

        storage.write_text(filename, content, encoding=encoding)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



from textwrap import wrap

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("wrap_text",)


def wrap_text(text: str, wrap_chars_after: int) -> str:
    return "\n".join(
        wrap(
            text=text,
            width=wrap_chars_after,
            replace_whitespace=False,
            # drop_whitespace=False,
        )
    )



import tempfile
import unittest
from pathlib import Path
from typing import Any, Dict, Type

# import pytest
from faker import Faker
from parametrize import parametrize
from pathy import use_fs, use_fs_cache

from ..storages.aws_s3 import AWSS3Storage
from ..storages.azure_cloud_storage import AzureCloudStorage
from ..storages.base import BaseStorage
from ..storages.cloud import CloudStorage, PathyFileSystemStorage
from ..storages.filesystem import FileSystemStorage
from ..storages.google_cloud_storage import GoogleCloudStorage

# from unittest.mock import patch


__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("TestStoragesTestCase",)

FAKER = Faker()


# class _GoogleCloudCredentials:
#     token = "1234"
#     project_id = "1234"


class TestStoragesTestCase(unittest.TestCase):
    """Test storages."""

    @parametrize(
        "storage_cls, kwargs, prefix, extension",
        [
            # FileSystemStorage
            (
                FileSystemStorage,
                {},
                "zzz",
                "docx",
            ),
            # PathyFileSystemStorage
            (
                PathyFileSystemStorage,
                {
                    "bucket_name": "testing",
                    "rel_path": "tmp",
                },
                "zzz",
                "docx",
            ),
            # AWS S3
            (
                AWSS3Storage,
                {
                    "bucket_name": "testing",
                    "rel_path": "tmp",
                    "credentials": {
                        "key_id": "key",
                        "key_secret": "key_secret",
                    },
                },
                "zzz",
                "docx",
            ),
            # Google Cloud Storage
            (
                GoogleCloudStorage,
                {
                    "bucket_name": "testing",
                    "rel_path": "tmp",
                    # "credentials": {
                    #     "json_file_path": GCS_CREDENTIALS,
                    # },
                },
                "zzz",
                "docx",
            ),
            # Azure Cloud Storage
            (
                AzureCloudStorage,
                {
                    "bucket_name": "testing",
                    "rel_path": "tmp",
                    "credentials": {"connection_string": "abcd1234"},
                },
                "zzz",
                "docx",
            ),
        ],
    )
    def test_storage(
        self,
        storage_cls: Type[BaseStorage],
        kwargs: Dict[str, Any],
        prefix: str,
        extension: str,
    ):
        """Test storage."""
        # Just for testing purposes
        if issubclass(storage_cls, CloudStorage):
            use_fs(Path(tempfile.gettempdir()))
            use_fs_cache()

        storage = storage_cls(**kwargs)
        # Text file
        file_text = storage.generate_filename(
            prefix=prefix, extension=extension
        )
        # Write to the text file
        text_result = storage.write_text(file_text, "Lorem ipsum")
        # Check if file exists
        self.assertTrue(storage.exists(file_text))
        # Assert correct return value
        self.assertIsInstance(text_result, int)

        # Bytes
        file_bytes = storage.generate_filename(
            prefix=prefix, extension=extension
        )
        # Write to bytes file
        bytes_result = storage.write_bytes(file_bytes, b"Lorem ipsum")
        # Check if file exists
        self.assertTrue(storage.exists(file_bytes))
        # Assert correct return value
        self.assertIsInstance(bytes_result, int)

    @parametrize(
        "storage_cls, kwargs, prefix, extension",
        [
            # FileSystemStorage
            (
                FileSystemStorage,
                {},
                "zzz",
                "",
            ),
            # PathyFileSystemStorage
            (
                PathyFileSystemStorage,
                {
                    "bucket_name": "testing",
                    "rel_path": "tmp",
                },
                "zzz",
                "",
            ),
        ],
    )
    def test_storage_generate_filename_exceptions(
        self,
        storage_cls: Type[BaseStorage],
        kwargs: Dict[str, Any],
        prefix: str,
        extension: str,
    ):
        """Test storage `generate_filename` exceptions."""
        storage = storage_cls(**kwargs)

        with self.assertRaises(Exception):
            # Generate filename
            storage.generate_filename(prefix=prefix, extension=extension)

    @parametrize(
        "storage_cls, kwargs",
        [
            # CloudStorage
            (CloudStorage, {"bucket_name": "testing"}),
        ],
    )
    def test_storage_initialization_exceptions(
        self,
        storage_cls: Type[BaseStorage],
        kwargs: Dict[str, Any],
    ):
        """Test storage initialization exceptions."""
        with self.assertRaises(Exception):
            # Initialize the storage
            storage_cls(**kwargs)

    @parametrize(
        "method_name, method_kwargs",
        [
            ("generate_filename", {"prefix": "zzz", "extension": "txt"}),
            ("write_text", {"filename": "test.txt", "data": "Test"}),
            ("write_bytes", {"filename": "test.txt", "data": b"Test"}),
            ("exists", {"filename": "test.txt"}),
            ("relpath", {"filename": "test.txt"}),
            ("abspath", {"filename": "test.txt"}),
        ],
    )
    def test_base_storage_exceptions(self, method_name, method_kwargs):
        """Test Base storage exceptions."""
        base_storage = BaseStorage()
        method = getattr(base_storage, method_name)
        with self.assertRaises(NotImplementedError):
            method(**method_kwargs)

    @parametrize(
        "method_name, method_kwargs",
        [
            ("authenticate", {}),
        ],
    )
    def test_cloud_storage_exceptions(self, method_name, method_kwargs):
        """Test Base storage exceptions."""

        class TestCloudStorage(CloudStorage):
            schema: str = "file"

        test_storage = TestCloudStorage(bucket_name="testing")
        method = getattr(test_storage, method_name)
        with self.assertRaises(NotImplementedError):
            method(**method_kwargs)

    def test_file_system_storage_abspath(self):
        """Test `FileSystemStorage` `abspath`."""
        storage = FileSystemStorage(
            root_path=tempfile.gettempdir(),
            rel_path="rel_tmp",
        )
        filename = storage.generate_filename(prefix="", extension="tmp")
        self.assertTrue(storage.abspath(filename).startswith("/tmp/rel_tmp/"))

    def test_pathy_file_system_storage_abspath(self):
        """Test `PathyFileSystemStorage` `abspath`."""
        storage = PathyFileSystemStorage(
            bucket_name="faker-file-tmp",
            root_path="root_tmp",
            rel_path="rel_tmp",
        )
        filename = storage.generate_filename(prefix="", extension="tmp")
        self.assertTrue(
            storage.abspath(filename).startswith(
                "file://faker-file-tmp/root_tmp/rel_tmp/"
            )
        )

    # @patch(
    #     "faker_file.storages.google_cloud_storage.service_account."
    #     "Credentials.from_service_account_file",
    #     new_callable=lambda: lambda __x: _GoogleCloudCredentials(),
    # )
    # @pytest.mark.xfail
    # def test_google_cloud_storage_authentication(self, func):
    #     """Test `GoogleCloudStorage` authentication."""
    #     GoogleCloudStorage(
    #         bucket_name="testing",
    #         rel_path="tmp",
    #         credentials={
    #             "json_file_path": "/i/dont/exist.json",
    #         },
    #     )



from io import BytesIO
from typing import Optional, Union, overload

from faker.providers import BaseProvider
from pptx import Presentation
from pptx.util import Inches

from ..base import BytesValue, FileMixin, StringValue
from ..constants import DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PptxFileProvider",)


class PptxFileProvider(BaseProvider, FileMixin):
    """PPTX file provider.

        Usage example:

        from faker_file.providers.pptx_file import PptxFileProvider

        file = PptxFileProvider(None).pptx_file()

    Usage example with options:

        from faker_file.providers.pptx_file import PptxFileProvider

        file = PptxFileProvider(None).pptx_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = PptxFileProvider(Faker()).pptx_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "pptx"

    @overload
    def pptx_file(
        self: "PptxFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue:
        ...

    @overload
    def pptx_file(
        self: "PptxFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        ...

    def pptx_file(
        self: "PptxFileProvider",
        storage: Optional[BaseStorage] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )

        data = {"content": content, "filename": filename}

        stream = BytesIO()
        prs = Presentation()
        prs.slide_width = Inches(25)
        prs.slide_height = Inches(16)
        # Make a blank slide with a text box with random text
        slide = prs.slides.add_slide(prs.slide_layouts[6])
        text_box = slide.shapes.add_textbox(0, 0, Inches(25), Inches(16))
        text_box.text = content
        text_box.text_frame.word_wrap = True
        prs.save(stream)

        if raw:
            raw_content = BytesValue(stream.getvalue())
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, stream.getvalue())

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        return file_name



import logging

import pdfkit

from ....constants import DEFAULT_FILE_ENCODING
from ...base.pdf_generator import BasePdfGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PdfkitPdfGenerator",)

LOGGER = logging.getLogger(__name__)


class PdfkitPdfGenerator(BasePdfGenerator):
    """Pdfkit PDF generator.

    Usage example:

        from faker import Faker
        from faker_file.providers.pdf_file import PdfFileProvider
        from faker_file.providers.pdf_file.generators import pdfkit_generator

        FAKER = Faker()

        file = PdfFileProvider(FAKER).pdf_file(
            pdf_generator_cls=pdfkit_generator.PdfkitPdfGenerator
        )
    """

    encoding: str = DEFAULT_FILE_ENCODING

    def handle_kwargs(self: "PdfkitPdfGenerator", **kwargs) -> None:
        """Handle kwargs."""
        if "encoding" in kwargs:
            self.encoding = kwargs["encoding"]

    def generate(self: "PdfkitPdfGenerator", content: str, **kwargs) -> bytes:
        """Generate PDF."""
        options = {"quiet": ""}
        if self.encoding is not None:
            options["encoding"] = self.encoding

        return pdfkit.from_string(
            f"<pre style='white-space: pre-wrap;'>{content}</pre>",
            options=options,
        )



import os
import os.path as op

from flask import Flask, request, session
from flask_babelex import Babel
from flask_sqlalchemy import SQLAlchemy

from .data import build_sample_db

app: Flask
db: SQLAlchemy


def create_app(config_filename: str) -> Flask:
    global app
    app = Flask(__name__)
    app.config.from_pyfile(config_filename)

    global db
    db = SQLAlchemy(app)

    with app.app_context():
        # Build a sample db on the fly, if one does not exist yet.
        app_dir = op.join(
            op.realpath(os.path.dirname(__file__)), "faker_file_admin"
        )
        database_path = op.join(app_dir, app.config["DATABASE_FILE"])
        if not os.path.exists(database_path):
            build_sample_db(db)

    return app


create_app("config.py")


# Initialize babel
babel = Babel(app)


@babel.localeselector
def get_locale():
    override = request.args.get("lang")

    if override:
        session["lang"] = override

    return session.get("lang", "en")


import faker_file_admin.main  # noqa



import os.path
import tempfile
import unittest
from copy import deepcopy
from typing import Any, Dict, List, Optional, Tuple, Type, Union

import tika
from faker import Faker
from parametrize import parametrize
from pathy import use_fs

from ..base import DEFAULT_REL_PATH
from ..providers.augment_file_from_dir import AugmentFileFromDirProvider
from ..providers.augment_file_from_dir.augmenters.nlpaug_augmenter import (
    DEFAULT_ACTION,
    DEFAULT_MODEL_PATH,
    ContextualWordEmbeddingsAugmenter,
)
from ..providers.base.text_augmenter import BaseTextAugmenter
from ..providers.base.text_extractor import BaseTextExtractor
from ..providers.docx_file import DocxFileProvider
from ..providers.eml_file import EmlFileProvider
from ..providers.epub_file import EpubFileProvider
from ..providers.pdf_file import PdfFileProvider
from ..providers.rtf_file import RtfFileProvider
from ..providers.txt_file import TxtFileProvider
from ..storages.base import BaseStorage
from ..storages.cloud import PathyFileSystemStorage
from ..storages.filesystem import FileSystemStorage
from .texts import TEXT_DOCX, TEXT_EML, TEXT_EPUB, TEXT_PDF, TEXT_RTF, TEXT_TXT

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("AugmentFileFromDirProviderTestCase",)


FileProvider = Union[
    DocxFileProvider,
    EmlFileProvider,
    EpubFileProvider,
    PdfFileProvider,
    RtfFileProvider,
    TxtFileProvider,
]

_FAKER = Faker()
FS_STORAGE = FileSystemStorage()
PATHY_FS_STORAGE = PathyFileSystemStorage(bucket_name="tmp", rel_path="tmp")
SOURCE_DIR_PATH = os.path.join(tempfile.gettempdir(), DEFAULT_REL_PATH)


class AugmentFileFromDirProviderTestCase(unittest.TestCase):
    """AugmentFileFromDirProvider test case."""

    def setUp(self: "AugmentFileFromDirProviderTestCase"):
        super().setUp()
        tika.initVM()
        use_fs(tempfile.gettempdir())

    @classmethod
    def setUpClass(cls: Type["AugmentFileFromDirProviderTestCase"]):
        super().setUpClass()
        DocxFileProvider(_FAKER).docx_file(
            prefix="source_",
            content=TEXT_DOCX,
            storage=FS_STORAGE,
        )
        EmlFileProvider(_FAKER).eml_file(
            prefix="source_",
            content=TEXT_EML,
            storage=FS_STORAGE,
        )
        EpubFileProvider(_FAKER).epub_file(
            prefix="source_",
            content=TEXT_EPUB,
            storage=FS_STORAGE,
        )
        PdfFileProvider(_FAKER).pdf_file(
            prefix="source_",
            content=TEXT_PDF,
            storage=FS_STORAGE,
        )
        RtfFileProvider(_FAKER).rtf_file(
            prefix="source_",
            content=TEXT_RTF,
            storage=FS_STORAGE,
        )
        TxtFileProvider(_FAKER).txt_file(
            prefix="source_",
            content=TEXT_TXT,
            storage=FS_STORAGE,
        )

    FAKER: Faker
    # provider, method_name, kwargs, storage
    __PARAMETRIZED_DATA: List[
        Tuple[
            Type[AugmentFileFromDirProvider],
            str,
            Dict[str, Any],
            Optional[Union[bool, PathyFileSystemStorage]],
        ]
    ] = [
        # AugmentFileFromDirProvider
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
                "text_augmenter_cls": ContextualWordEmbeddingsAugmenter,
                "text_augmenter_kwargs": {
                    "model_path": DEFAULT_MODEL_PATH,
                    "action": DEFAULT_ACTION,
                },
            },
            None,
        ),
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
            },
            None,
        ),
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
            },
            False,
        ),
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
            },
            PATHY_FS_STORAGE,
        ),
        # Testing Nones
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
                "text_extractor_cls": None,
                "text_augmenter_cls": None,
            },
            PATHY_FS_STORAGE,
        ),
        # DOCX
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["docx"]},
            None,
        ),
        # EML
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["eml"]},
            None,
        ),
        # EPUB
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["epub"]},
            None,
        ),
        # PDF
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["pdf"]},
            None,
        ),
        # RTF
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["rtf"]},
            None,
        ),
        # TXT
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {"source_dir_path": SOURCE_DIR_PATH, "extensions": ["txt"]},
            None,
        ),
    ]

    # provider, method_name, kwargs, storage
    __RAW_PARAMETRIZED_DATA: List[
        Tuple[
            Type[AugmentFileFromDirProvider],
            str,
            Dict[str, Any],
            Optional[Union[bool, PathyFileSystemStorage]],
        ]
    ] = [
        # AugmentFileFromDirProvider
        (
            AugmentFileFromDirProvider,
            "augment_file_from_dir",
            {
                "source_dir_path": SOURCE_DIR_PATH,
            },
            None,
        ),
    ]

    @parametrize(
        "provider, method_name, kwargs, storage",
        __PARAMETRIZED_DATA,
    )
    def test_standalone(
        self: "AugmentFileFromDirProviderTestCase",
        provider: Type[AugmentFileFromDirProvider],
        method_name: str,
        kwargs: Dict[str, Any],
        storage: Optional[BaseStorage] = None,
    ) -> None:
        """Test standalone."""
        _kwargs = deepcopy(kwargs)
        if storage is False:
            storage = FS_STORAGE
        _provider = provider(None)  # noqa
        _method = getattr(_provider, method_name)
        _kwargs["storage"] = storage
        _file = _method(**_kwargs)
        self.assertTrue((storage or FS_STORAGE).exists(_file))

    def test_augment_file_from_dir_extract_not_implemented_exception(
        self: "AugmentFileFromDirProviderTestCase",
    ):
        with self.assertRaises(NotImplementedError):
            AugmentFileFromDirProvider(_FAKER).augment_file_from_dir(
                source_dir_path=SOURCE_DIR_PATH,
                text_extractor_cls=BaseTextExtractor,
            )

        class MyTextExtractor(BaseTextExtractor):
            """Test text extractor."""

        with self.assertRaises(NotImplementedError):
            AugmentFileFromDirProvider(_FAKER).augment_file_from_dir(
                source_dir_path=SOURCE_DIR_PATH,
                text_extractor_cls=MyTextExtractor,
            )

    def test_augment_file_from_dir_augment_not_implemented_exception(
        self: "AugmentFileFromDirProviderTestCase",
    ):
        with self.assertRaises(NotImplementedError):
            AugmentFileFromDirProvider(_FAKER).augment_file_from_dir(
                source_dir_path=SOURCE_DIR_PATH,
                text_augmenter_cls=BaseTextAugmenter,
            )

        class MyTextAugmenter(BaseTextAugmenter):
            """Test text augmenter."""

        with self.assertRaises(NotImplementedError):
            AugmentFileFromDirProvider(_FAKER).augment_file_from_dir(
                source_dir_path=SOURCE_DIR_PATH,
                text_augmenter_cls=MyTextAugmenter,
            )

    @parametrize(
        "provider, method_name, kwargs, storage",
        __RAW_PARAMETRIZED_DATA,
    )
    def test_raw_standalone(
        self: "AugmentFileFromDirProviderTestCase",
        provider: Type[AugmentFileFromDirProvider],
        method_name: str,
        kwargs: Dict[str, Any],
        storage: Optional[BaseStorage] = None,
    ) -> None:
        """Test standalone."""
        _kwargs = deepcopy(kwargs)
        if storage is False:
            storage = FS_STORAGE
        _provider = provider(None)  # noqa
        _method = getattr(_provider, method_name)
        _kwargs["storage"] = storage
        _kwargs["raw"] = True
        _bytes = _method(**_kwargs)
        self.assertIsInstance(_bytes, bytes)
        self.assertGreater(len(_bytes), 0)
